@@ -1,6 +1,6 @@
-/* Compute look-ahead criteria for bison,
+/* Compute lookahead criteria for bison,
 
-   Copyright (C) 1984, 1986, 1989, 2000, 2002, 2004 Free Software
+   Copyright (C) 1984, 1986, 1989, 2000, 2002, 2004, 2006 Free Software
    Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
@@ -17,8 +17,8 @@
 
    You should have received a copy of the GNU General Public License
    along with Bison; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #ifndef LALR_H_
 # define LALR_H_
@@ -32,35 +32,41 @@
 /* Import the definition of CORE, TRANSITIONS and REDUCTIONS. */
 # include "state.h"
 
-/* Compute how to make the finite state machine deterministic; find
-   which rules need look-ahead in each state, and which look-ahead
-   tokens they accept.  */
 
+/** Build the LALR(1) automaton.
+
+   Compute how to make the finite state machine deterministic; find
+   which rules need lookahead in each state, and which lookahead
+   tokens they accept.
+
+   Builds:
+   - #goto_map
+   - #from_state
+   - #to_state
+*/
 void lalr (void);
 
-/* Release the information related to look-ahead tokens.  Can be performed
-   once the action tables are computed.  */
 
+/** Release the information related to lookahead tokens.
+
+   Can be performed once the action tables are computed.  */
 void lalr_free (void);
 
 
-/* lalr() builds these data structures. */
-
-/* GOTO_MAP, FROM_STATE and TO_STATE -- record each shift transition
-   which accepts a variable (a nonterminal).
-
-   FROM_STATE[T] -- state number which a transition leads from.
-   TO_STATE[T] -- state number it leads to.
-
-   All the transitions that accept a particular variable are grouped
-   together and GOTO_MAP[I - NTOKENS] is the index in FROM_STATE and
-   TO_STATE of the first of them.  */
-
 typedef size_t goto_number;
 # define GOTO_NUMBER_MAXIMUM ((goto_number) -1)
 
+/** Index into #from_state and #to_state.
+
+   All the transitions that accept a particular variable are grouped
+   together and GOTO_MAP[I - NTOKENS] is the index in FROM_STATE and
+   TO_STATE of the first of them.  */
 extern goto_number *goto_map;
+
+/** State number which a transition leads from. */
 extern state_number *from_state;
+
+/** State number it leads to.  */
 extern state_number *to_state;
 
 
